###############################################################################
#
# The gcc versioning information.  In a sed command below, the specfile winds
# pre-release version numbers in BASE-VER back to the last actually-released
# number.
%global DATE 20220507
%global gitrev fa107326a13af9a7d7aa0df28fe364db0f6fb171
%global gcc_version 12.1.1
%global gcc_major 12
%global gcc_release 1

%global bpf_binutils_version 2.38-1
%global isl_version 0.16.1
%global isl_libmajor 15

%global _performance_build 1
# Hardening slows the compiler way too much.
%undefine _hardened_build
%if 0%{?fedora} > 27 || 0%{?rhel} >= 8
# Until annobin is fixed (#1519165).
%undefine _annotated_build
%endif

Summary: BPF C compiler
Name: bpf-gcc
Version: %{gcc_version}
Release: %{gcc_release}%{?dist}
# libgcc, libgfortran, libmudflap, libgomp, libstdc++ and crtstuff have
# GCC Runtime Exception.
License: GPLv3+ and GPLv3+ with exceptions and GPLv2+ with exceptions and LGPLv2+ and BSD
URL: http://gcc.gnu.org
BuildRequires:  gcc-c++
#BuildRequires: isl-devel >= %{isl_version}

# The source for this package was pulled from upstream's vcs.  Use the
# following commands to generate the tarball:
# svn export svn://gcc.gnu.org/svn/gcc/branches/redhat/gcc-8-branch@%%{SVNREV} gcc-%%{version}-%%{DATE}
# tar cf - gcc-%%{version}-%%{DATE} | xz -9e > gcc-%%{version}-%%{DATE}.tar.xz
%global srcdir gcc-%{version}-%{DATE}
Source0: %{srcdir}.tar.xz

Patch0: gcc12-hack.patch
Patch2: gcc12-sparc-config-detection.patch
Patch3: gcc12-libgomp-omp_h-multilib.patch
Patch4: gcc12-libtool-no-rpath.patch
Patch5: gcc12-isl-dl.patch
Patch6: gcc12-isl-dl2.patch
Patch7: gcc12-libstdc++-docs.patch
Patch8: gcc12-no-add-needed.patch
Patch9: gcc12-Wno-format-security.patch
Patch10: gcc12-rh1574936.patch
Patch11: gcc12-d-shared-libphobos.patch

Patch900: cross-intl-filename.patch
Patch901: cross-gcc-format-config.patch

#----------------------- Oracle Patches -------------------------------------

# BPF and BTF backports
Patch10000: cross-gcc12-bpf.patch

%if 0%{?fedora} >= 29 || 0%{?rhel} > 7
BuildRequires: binutils >= 2.30-17
%else
BuildRequires: binutils >= 2.24
%endif
BuildRequires: zlib-devel, gettext, dejagnu, bison, flex, texinfo, sharutils
BuildRequires: binutils-bpf-unknown-none >= %{bpf_binutils_version}

# right now, we only have this for arm
#BuildRequires: glibc-arm-linux-gnu

# Make sure pthread.h doesn't contain __thread tokens
# Make sure glibc supports stack protector
# Make sure glibc supports DT_GNU_HASH
BuildRequires: glibc-devel >= 2.4.90-13
BuildRequires: elfutils-devel >= 0.147
BuildRequires: elfutils-libelf-devel >= 0.147
BuildRequires: gmp-devel >= 4.1.2-8, mpfr-devel >= 2.2.1, libmpc-devel >= 0.8.1
Provides: bundled(libiberty)

%description
BPF cross GNU C compiler collection.

###############################################################################
#
# Packages definition
#
###############################################################################

%package -n gcc-bpf-unknown-none
Summary: Cross GCC for bpf-unknown-none
BuildRequires: binutils-bpf-unknown-none >= %{bpf_binutils_version}
Requires: binutils-bpf-unknown-none >= %{bpf_binutils_version}
#%if 0%{?__isa_bits} == 64
#Requires: libisl.so.%{isl_libmajor}()(64bit)
#%else
#Requires: libisl.so.%{isl_libmajor}
#%endif
%description -n gcc-bpf-unknown-none
Cross-build BPF GNU C compiler.

###############################################################################
#
# Preparation
#
###############################################################################
%prep

%setup -q -n %{srcdir} -c
cd %{srcdir}
%patch0 -p0 -b .hack~
%patch2 -p0 -b .sparc-config-detection~
%patch3 -p0 -b .libgomp-omp_h-multilib~
%patch4 -p0 -b .libtool-no-rpath~
#%patch5 -p0 -b .isl-dl
#%patch6 -p0 -b .isl-dl2
%patch7 -p0 -b .libc++docs
%patch8 -p0 -b .no-add-needed~
%patch9 -p0 -b .foffload-default~
%patch10 -p0 -b .Wno-format-security~
%patch11 -p0 -b .libphobos

#%patch900 -p0 -b .cross-intl~
%patch901 -p0 -b .format-config~

%patch10000 -p1 -b .bpf~

echo 'BPF GCC cross %{version}-%{gcc_release}' > gcc/DEV-PHASE

echo 'TM_H += $(srcdir)/config/rs6000/rs6000-modes.h' >> gcc/config/rs6000/t-rs6000

./contrib/gcc_update --touch

LC_ALL=C sed -i -e 's/\xa0/ /' gcc/doc/options.texi

sed -i -e 's/Common Driver Var(flag_report_bug)/& Init(1)/' gcc/common.opt

# This test causes fork failures, because it spawns way too many threads
rm -f gcc/testsuite/go.test/test/chan/goroutines.go

cd ..

###############################################################################
#
# Build
#
###############################################################################
%build

%global builddir %{_builddir}/%{srcdir}

# Undo the broken autoconf change in recent Fedora versions
export CONFIG_SITE=NONE

CC=gcc
CXX=g++
OPT_FLAGS=`echo %{optflags}|sed -e 's/\(-Wp,\)\?-D_FORTIFY_SOURCE=[12]//g'`
OPT_FLAGS=`echo $OPT_FLAGS|sed -e 's/-m64//g;s/-m32//g;s/-m31//g'`
OPT_FLAGS=`echo $OPT_FLAGS|sed -e 's/-mfpmath=sse/-mfpmath=sse -msse2/g'`
OPT_FLAGS=`echo $OPT_FLAGS|sed -e 's/ -pipe / /g'`
%ifarch sparc
OPT_FLAGS=`echo $OPT_FLAGS|sed -e 's/-mcpu=ultrasparc/-mtune=ultrasparc/g;s/-mcpu=v[78]//g'`
%endif
%ifarch %{ix86}
OPT_FLAGS=`echo $OPT_FLAGS|sed -e 's/-march=i.86//g'`
%endif
OPT_FLAGS=`echo $OPT_FLAGS|sed -e 's/-Werror=format-security/-Wformat-security/g'`
OPT_FLAGS=`echo "$OPT_FLAGS" | sed -e 's/[[:blank:]]\+/ /g'`
case "$OPT_FLAGS" in
  *-fasynchronous-unwind-tables*)
    sed -i -e 's/-fno-exceptions /-fno-exceptions -fno-asynchronous-unwind-tables /' \
      %{srcdir}/libgcc/Makefile.in
    ;;
esac

#
# Configure the compiler
#
cd %{builddir}

echo "=== CONFIGURING bpf-unknown-none"

arch=bpf-unknown-none
prefix=$arch-
build_dir=bpf-unknown-none
target=bpf-unknown-none

echo $arch: target is $target

CONFIG_FLAGS="--disable-shared"

mkdir $build_dir
cd $build_dir

# We could optimize the cross builds size by --enable-shared but the produced
# binaries may be less convenient in the embedded environment.
CC="$CC" \
CXX="$CXX" \
CFLAGS="$OPT_FLAGS" \
CXXFLAGS="`echo " $OPT_FLAGS " | sed 's/ -Wall / /g;s/ -fexceptions / /g' \
	  | sed 's/ -Wformat-security / -Wformat -Wformat-security /'`" \
CFLAGS_FOR_TARGET="-g -O2 -Wall -fexceptions" \
AR_FOR_TARGET=%{_bindir}/$arch-ar \
AS_FOR_TARGET=%{_bindir}/$arch-as \
DLLTOOL_FOR_TARGET=%{_bindir}/$arch-dlltool \
LD_FOR_TARGET=%{_bindir}/$arch-ld \
NM_FOR_TARGET=%{_bindir}/$arch-nm \
OBJDUMP_FOR_TARGET=%{_bindir}/$arch-objdump \
RANLIB_FOR_TARGET=%{_bindir}/$arch-ranlib \
READELF_FOR_TARGET=%{_bindir}/$arch-readelf \
STRIP_FOR_TARGET=%{_bindir}/$arch-strip \
WINDRES_FOR_TARGET=%{_bindir}/$arch-windres \
WINDMC_FOR_TARGET=%{_bindir}/$arch-windmc \
LDFLAGS='-Wl,-z,relro ' \
../%{srcdir}/configure \
	--bindir=%{_bindir} \
	--build=%{_target_platform} \
	--datadir=%{_datadir} \
	--disable-decimal-float \
	--disable-dependency-tracking \
	--disable-gold \
	--disable-libgcj \
	--disable-libgomp \
	--disable-libmpx \
	--disable-libquadmath \
	--disable-libssp \
	--disable-libunwind-exceptions \
	--disable-silent-rules \
	--disable-sjlj-exceptions \
	--disable-threads \
	--with-ld=/usr/bin/$arch-ld \
	--enable-__cxa_atexit \
	--enable-checking=release \
	--enable-gnu-unique-object \
	--enable-initfini-array \
	--enable-languages=c \
	--enable-linker-build-id \
	--enable-lto \
	--enable-nls \
	--enable-obsolete \
	--enable-plugin \
	--enable-targets=all \
	--exec-prefix=%{_exec_prefix} \
	--host=%{_target_platform} \
	--includedir=%{_includedir} \
	--infodir=%{_infodir} \
	--libexecdir=%{_libexecdir} \
	--localstatedir=%{_localstatedir} \
	--mandir=%{_mandir} \
	--prefix=%{_prefix} \
	--program-prefix=$prefix \
	--sbindir=%{_sbindir} \
	--sharedstatedir=%{_sharedstatedir} \
	--sysconfdir=%{_sysconfdir} \
	--target=$target \
	--with-bugurl=http://bugzilla.redhat.com/bugzilla/ \
	--with-gcc-major-version-only \
	--without-isl \
	--with-newlib \
	--with-plugin-ld=%{_bindir}/$arch-ld \
	--with-sysroot=%{_prefix}/$arch/sys-root \
	--with-system-libunwind \
	--with-system-zlib \
	--without-headers \
	$CONFIG_FLAGS
cd ..

#
# Build the compiler
#

echo "=== BUILDING bpf-unknown-none"

arch=bpf-unknown-none
build_dir=bpf-unknown-none

BUILD_FLAGS=

AR_FOR_TARGET=%{_bindir}/$arch-ar \
AS_FOR_TARGET=%{_bindir}/$arch-as \
DLLTOOL_FOR_TARGET=%{_bindir}/$arch-dlltool \
LD_FOR_TARGET=%{_bindir}/$arch-ld \
NM_FOR_TARGET=%{_bindir}/$arch-nm \
OBJDUMP_FOR_TARGET=%{_bindir}/$arch-objdump \
RANLIB_FOR_TARGET=%{_bindir}/$arch-ranlib \
READELF_FOR_TARGET=%{_bindir}/$arch-readelf \
STRIP_FOR_TARGET=%{_bindir}/$arch-strip \
WINDRES_FOR_TARGET=%{_bindir}/$arch-windres \
WINDMC_FOR_TARGET=%{_bindir}/$arch-windmc \
make -C $build_dir %{_smp_mflags} tooldir=%{_prefix} $BUILD_FLAGS all-gcc

echo "=== BUILDING LIBGCC bpf-unknown-none"
make -C $build_dir tooldir=%{_prefix} all-target-libgcc

#
# Installation
#
%install
rm -rf %{buildroot}

mkdir -p %{buildroot}%{_prefix}/bpf-unknown-none/sys-root
echo "=== INSTALLING bpf-unknown-none"

arch=bpf-unknown-none
cpu=${arch%%%%-*}

make -C $arch DESTDIR=%{buildroot} install-gcc install-target-libgcc

# Drop the documentation.
rm -rf %{buildroot}%{_infodir}

# Remove binaries we will not be including, so that they don't end up in
# bpf-gcc-debuginfo
rm -f %{buildroot}%{_libdir}/{libffi*,libiberty.a}
rm -f %{buildroot}%{_libexecdir}/gcc/*/%{gcc_major}/install-tools/{mkheaders,fixincl}
rm -f %{buildroot}%{_prefix}/bin/*-gcc-%{gcc_major} || :
rmdir  %{buildroot}%{_includedir}

find %{buildroot}%{_datadir} -name gcc.mo |
while read x
do
    y=`dirname $x`
    mv $x $y/bpf-gcc.mo
done

%find_lang bpf-gcc

gzip %{buildroot}%{_mandir}/man1/*.1
rm %{buildroot}%{_mandir}/man7/*.7
rmdir %{buildroot}%{_mandir}/man7

# # Add manpages the additional symlink-only targets
# %if %{build_powerpc}%{build_powerpc64}%{build_powerpc64le}
# for i in powerpc*
# do
#     ln -s $i ppc${i#powerpc}
# done
# %endif

#cd -

#
# Build the file lists
#

arch=bpf-unknown-none
cpu=${arch%%%%-*}

(
  echo '%{_bindir}/'$arch'*-cpp'
  echo '%{_bindir}/'$arch'*-gcc'
  echo '%{_bindir}/'$arch'*-gcc-ar'
  echo '%{_bindir}/'$arch'*-gcc-nm'
  echo '%{_bindir}/'$arch'*-gcc-ranlib'
  # echo '%{_bindir}/'$arch'*-gcov*' #gcov is not built for bpf
  echo '%{_bindir}/'$arch'*-lto-dump'
  echo '%{_mandir}/man1/'$arch'*-cpp*'
  echo '%{_mandir}/man1/'$arch'*-gcc*'
  echo '%{_mandir}/man1/'$arch'*-gcov*'
  echo '%{_mandir}/man1/'$arch'*-lto-dump*'
  echo '/usr/lib/gcc/'$cpu'-*/'
  echo '%{_libexecdir}/gcc/'$cpu'*/*/cc1'
  echo '%{_libexecdir}/gcc/'$cpu'*/*/collect2'
  echo '%{_libexecdir}/gcc/'$cpu'*/*/[abd-z]*'
  echo %{_prefix}/$arch/sys-root
) >files.$arch

%global __ar_no_strip $RPM_BUILD_DIR/%{srcdir}/ar-no-strip
cat >%{__ar_no_strip} <<EOF
#!/bin/bash
f=\$2
if [ \${f##*/} = libgcc.a -o \${f##*/} = libgcov.a ]
then
	:
else
	%{__strip} \$*
fi
EOF
chmod +x %{__ar_no_strip}
%undefine __strip
%global __strip %{__ar_no_strip}

###############################################################################
#
# Filesets
#
###############################################################################

%files -n gcc-bpf-unknown-none -f files.bpf-unknown-none -f bpf-gcc.lang
%doc %{srcdir}/COPYING*
%doc %{srcdir}/README

%changelog
* Tue Feb 20 2024 David Faust <david.faust@oracle.com> - 12.1.1-1
- Upgrade to GCC 12.1.1.
- Backport the following supporting commits from upstream:
    2c05a2d1a8e c: use CONST_DECL for enumerators in TYPE_VALUES
    f25efe50374 docs: Add @defbuiltin family of helpers
- Backport upstream BPF and BTF fixes since 12.1.1:
    f995f567730 bpf: fix zero_extendqidi2 ldx template
    1959aeee1e0 bpf: emit empty epilogues in naked functions
    f64448f4ff1 bpf: add constant pointer to helper-skb-ancestor-cgroup-id.c test
    bc77c035c45 Add -gno-strict-dwarf to dg-options in various btf enum tests
    dfc88fb51c1 bpf: Correct BTF for kernel_helper attributed decls
    4a5bb8bc1c5 btf: print string offset in comment
    1502d724df8 btf: change encoding of forward-declared enums [PR111735]
    b8cf266f4ca btf: avoid wrong DATASEC entries for extern vars [PR112849]
    b6abc5dbfa5 BTF: fix PR debug/112656
    9b6e2cb7e61 BTF: fix PR debug/112768
    b506834e7fa bpf: quote section names whenever necessary in assembly output
    72e212c0290 bpf: change ASM_COMMENT_START to '#'
    86903dd94e6 testsuite: move gcc.c-torture/compile/libcall-2.c to gcc.target/i386/libcall-1.c
    4ed0740c6e8 testsuite: fix gcc.c-torture/compile/libcall-2.c in -m32
    880ae958fad varasm.cc: refer to assemble_external_libcall only ifdef ASM_OUTPUT_EXTERNAL
    f31a019d116 Emit funcall external declarations only if actually used.
    099b15e2bdb Fixed problem with BTF defining smaller enums.
    19cc5857e2c bpf: Forces __buildin_memcmp not to generate a call upto 1024 bytes.
    a789af1ca27 bpf: Corrected condition in core_mark_as_access_index.
    61e060c4dce bpf: Delayed the removal of the parser enum plugin handler.
    b6db325ac3e bpf: remove bpf-helpers.h
    8329ba35974 bpf: fix pseudo-c asm emitted for *mulsidi3_zeroextend
    9815ef77313 bpf: testsuite: fix expected regexp in gcc.target/bpf/ldxdw.c
    327d38f1018 bpf: Improvements in CO-RE builtins implementation.
    721f7e2c4e5 Fix tests for PR 106537.
    e2c42860b6b bpf: neg instruction does not accept an immediate
    35b5762a740 bpf: bump maximum frame size limit to 32767 bytes
    e1f45bea274 Add warning options -W[no-]compare-distinct-pointer-types
    b7c50f68f26 bpf: support `naked' function attributes in BPF targets
    5da4c0b85a9 bpf: remove useless define_insn for extendsisi2
    489e1adf779 bpf: fix pseudoc w regs for small modes [PR111029]
    f93a612fc45 bpf: liberate R9 for general register allocation
    6103df1e4fa bpf: allow exceeding max num of args in BPF when always_inline
    e7cae4f873f bpf: Fixed GC mistakes in BPF builtins code.
    c2a447d8404 bpf: CO-RE builtins support tests.
    e0a81559c19 bpf: Implementation of BPF CO-RE builtins
    9f5372707f9 bpf: disable tail call optimization in BPF targets
    14dab1a1bcc bpf: ISA V4 sign-extending move and load insns [PR110782,PR110784]
    9cbf4286a9d bpf: minor doc cleanup for command-line options
    31d18ff4424 bpf: correct pseudo-C template for add3 and sub3
    db189d02ef1 bpf: add v3 atomic instructions
    590a8287bf2 bpf: fix generation of neg and neg32 BPF instructions
    8d36a0f5952 bpf: don't print () in bpf_print_operand_address
    be16bb885ef bpf: add pseudo-c asm dialect for "nop"
    4be3919fb75 bpf: sdiv/smod are now part of BPF V4
    da445a58582 bpf: remove -mkernel option and BPF_KERNEL_VERSION_CODE
    f1e34551e4d bpf: make use of the bswap{16,32,64} V4 BPF instruction
    77d0f9ec380 bpf: pseudo-c assembly dialect support
    13ed106723c bpf: fixed template for neg (added second operand)
    53d12ecd624 bpf: enable instruction scheduling
    934da923a72 btf: fix bootstrap -Wformat errors [PR110073]
    f2e60a00c7c btf: Fix -Wformat errors
    7aae58b04b9 btf: improve -dA comments for testsuite
    00887865398 btf: be clear when record size/type is not used
    c348a717213 bpf: fix memory constraint of ldx/stx instructions [PR108790]
    70b303049e4 btf: correct generation for extern funcs [PR106773]
    4f7aa145b79 btf: fix 'extern const void' variables [PR106773]
    2bce22e88e1 btf: add 'extern' linkage for variables [PR106773]
    8422861bddc btf: Add support to BTF_KIND_ENUM64 type
    5df04a7aa83 btf: do not use the CHAR `encoding' bit for BTF
    32566720f3a btf: emit linkage information in BTF_KIND_FUNC entries
    3b81f5c4d8e bpf: disable -fstack-protector in BPF
    c7279270a2d bpf: correct bpf_print_operand for floats [PR108293]
    5ddfe794405 bpf: add define_insn for bswap
    6052482f841 bpf: avoid possible use of uninitialized variable
    564b1118465 bpf: Use enum for resolved overloaded builtins
    93ab7d03dfb bpf: cleanup missed refactor
    068baae1864 bpf: add preserve_field_info builtin
    2a2fb3e30c2 bpf: remove unused variables from bpf_expand_{prologue,epilogue}
    b504149d2c9 bpf: handle anonymous members in CO-RE reloc [PR106745]
    c68b5c078bb bpf: define __bpf__ as well as __BPF__ as a target macro
    6d1f144b3e6 bpf: facilitate constant propagation of function addresses
- Orabug 36268701
- Reviewed-by: Jose E. Marchesi <jose.marchesi@oracle.com>

* Tue Sep 15 2020 Jose E. Marchesi <jose.marchesi@oracle.com> - 8.3.1-5
- Backport of upstream patch:
  5bcc0fa05ef713594f6c6d55d5c837e13a9c9803 - Use expected insn for BPF NOP.
- Reviewed-by: David Faust <david.faust@oracle.com>

* Wed Sep 02 2020 Jose E. Marchesi <jose.marchesi@oracle.com> - 8.3.1-4
- Backports of upstream patches:
  + c9d44022359 bpf: use elfos.h
  + e87c540fe43 bpf: remove trailing whitespaces from source files
  + af30b83b509 bpf: more flexible support for kernel helpers
- Reviewed-by: Qing Zhao  <qing.zhao@oracle.com>

* Fri May 01 2020 Jose E. Marchesi <jose.marchesi@oracle.com> - 8.3.1-3
- Avoid generation of funcalls for one_cmpl{si,di}2 insns in BPF.
- Reviewed-by: Elena Zannoni <elena.zannoni@oracle.com>

* Tue Apr 21 2020 Jose E. Marchesi <jose.marchesi@oracle.com> - 8.3.1-2
- Support for the -mxbpf command line option.
- Do not save/restore callee-saved registers in function
  prologues/epilogues unless generating xBPF.
- Reviewed-by: Elena Zannoni <elena.zannoni@oracle.com>

* Wed Mar 11 2020 Jose E. Marchesi <jose.marchesi@oracle.com> - 8.3.1-1
- New package.
